const minimist = require("minimist");
const {parseArgs} = require("./parseArgs");
const {transferPlugin} = require("./transferPlugins");
const {transferSulu} = require("./transferSulu");
const chokidar = require("chokidar");
const fs = require("fs-extra");
const path = require("path");
const { tryReadFile } = require("./utils");

const manifestName = "buildManifest.json";

function tryReadManifest(src) {
  try {
    const manifest = fs.readJSONSync(path.join(src, manifestName)) || {};
    Object.values(manifest).forEach((val) => val.sourceModified = new Date(val.sourceModified));
    return manifest;
  } catch(e) {
    return {};
  }
}

async function main() {
  try {
    const args = minimist(process.argv.slice(2));

    const {
      suluDir,
      buildOutputPath,
      manifests,
      version,
      options: {
        allowPluginsToOverride,
        reloadOnChanges,
        incrementalBuild,
      },
      log
    } = parseArgs({
      writeFolder: args.writeFolder,
      suluDir: args.suluDir,
      writeDir: args.writeDir,
      pluginPaths: args.pluginPaths,
      version: args.version,
      _w: args.w,
      _v: args.v,
      _o: args.o,
      _s: args.s,
      _p: args.p,
      _t: args.t
    });

    const core = () => {
      const buildManifest = incrementalBuild ? tryReadManifest(buildOutputPath) : {};
      const revision = new Date().toString();
      const dotenv = [];
      if (version) {
        dotenv.push(`VITE_BUILD_VERSION=${version}`);
      }
      transferSulu({
        log,
        sourcePath: suluDir,
        buildOutputPath,
        buildManifest,
        dotenv,
        revision
      });

      manifests.forEach(manifest => {
        transferPlugin({
          manifest,
          buildOutputPath,
          allowPluginsToOverride,
          log,
          buildManifest,
          dotenv,
          revision
        });
      });

      Object.entries(buildManifest)
        .forEach(([fname, value]) => {
          if (!value.stillExists) {
            const toRemove = path.join(buildOutputPath, fname);
            console.log("Source for built file no longer exists, removing built", toRemove);
            fs.removeSync(toRemove);
            delete buildManifest[fname];
          } else {
            // no longer needed, but object needs to be clean for save
            delete value.stillExists;
          }
        });

      fs.writeJSONSync(path.join(buildOutputPath, manifestName), buildManifest);

      const existingDotenv = tryReadFile(path.join(buildOutputPath, ".env")).split("\n").filter(x => !!x);

      let mismatch = dotenv.length !== existingDotenv.length;
      dotenv.forEach((line,i) => {
        if (existingDotenv[i] !== line) {
          mismatch = true;
        }
      });

      if (mismatch) {
        log("Writing a new cumulated .env file");
        fs.writeFileSync(path.join(buildOutputPath, ".env"), dotenv.join("\n"));
      }

      // log(buildManifest);
    };

    core();

    if (reloadOnChanges) {
      const watcher = chokidar.watch(
        [suluDir, ...manifests.map(m => m.absolutePath)],
        {
          persistent: true,
          ignored: path => {
            const r = ["node_modules", ".git", "yarn.lock", "yarn-error.log"].find(x => path.includes(x));
            return !!r;
          }
        });
      let isReady = false;
      watcher.on("ready", () => {
        isReady = true;
      });
      watcher.on("change",  (eventType) => {
        console.log(`>>>>> Changes detected, reloading: ${eventType} <<<<<`);
        core();
      });
      watcher.on("add", (eventType) => {
        if (isReady) {
          console.log(`>>>>> Changes detected (file added), reloading: ${eventType} <<<<<`);
          core();
        }
      });
      watcher.on("unlink", (eventType) => {
        console.log(`>>>>> Changes detected (file removed), reloading: ${eventType} <<<<<`);
        core();
      });
    }
  } catch(e) {
    console.log("Error occured", e);
  }
}

main();
